//
//  MOPathMenuController.m
//  MOPathMenuController
//
//  Created by mo jun on 3/4/13.
//  Copyright (c) 2013 mo jun. All rights reserved.
//

#import "MOPathMenuController.h"

@interface MOPathMenuController ()

- (void)pan:(UIPanGestureRecognizer *)gesture;

- (void)tap:(UIPanGestureRecognizer *)gesture;

- (void)panAnimationDidFinishedCallback:(DDMenuPanCompletion)completion;

- (void)showLeftController:(BOOL)animated duration:(NSTimeInterval)interval;

- (void)showRightController:(BOOL)animated duration:(NSTimeInterval)interval;

- (void)showRootController:(BOOL)animated duration:(NSTimeInterval)interval;

- (void)showLeftController:(BOOL)animated;

- (void)showRightController:(BOOL)animated;

- (void)showRootController:(BOOL)animated;

- (void)animationDidStop:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context;

- (void)showShadow:(BOOL)val;

@end

@implementation MOPathMenuController

- (id)initWithRootViewController:(UIViewController *)controller
{
    self = [super initWithNibName:nil bundle:nil];
    if (self) {
        // Custom initialization
        _rootController = [controller retain];
        panMaskView = [[UIView alloc]init];
        panMaskView.hidden = YES;
        panMaskView.backgroundColor = [UIColor clearColor];
        
        UIPanGestureRecognizer *pan = [[[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(pan:)]autorelease];
        pan.delegate = (id<UIGestureRecognizerDelegate>)self;
        self.pan = pan;
        self.pan.enabled = YES;
        
        UIPanGestureRecognizer *maskPan = [[[UIPanGestureRecognizer alloc]initWithTarget:self action:@selector(pan:)]autorelease];
        maskPan.delegate = (id<UIGestureRecognizerDelegate>)self;
        self.maskPan = maskPan;
        self.maskPan.enabled = NO;
        
        UITapGestureRecognizer *tap = [[[UITapGestureRecognizer alloc]initWithTarget:self action:@selector(tap:)]autorelease];
        self.tap = tap;
        self.tap.enabled = NO;
        
        self.duration = 0.3f;
        self.isDisplayShadow = YES;
        
    }
    return self;
}

- (void)showShadow:(BOOL)val{
    if (!_rootController || !_isDisplayShadow) return;
    
    _rootController.view.layer.shadowOpacity = val ? 0.8f : 0.0f;
    if (val) {
        _rootController.view.layer.cornerRadius = 4.0f;
        _rootController.view.layer.shadowOffset = CGSizeZero;
        _rootController.view.layer.shadowRadius = 4.0f;
        _rootController.view.layer.shadowPath = [UIBezierPath bezierPathWithRect:self.view.bounds].CGPath;
    }
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view.
    
    UIView *rootView = _rootController.view;
    rootView.frame = self.view.bounds;
    [self.view addSubview:rootView];
    [rootView addGestureRecognizer:self.tap];
    [rootView addGestureRecognizer:self.pan];

    panMaskView.frame = rootView.bounds;
    [rootView addSubview:panMaskView];
    [panMaskView addGestureRecognizer:self.maskPan];

}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)dealloc{
    [_leftController release];
    [_rightController release];
    [_rootController release];
    
    [_pan release];
    [_tap release];
    [panMaskView release];
    [super dealloc];
}

#pragma mark - Setters

- (void)setDelegate:(id<MOPathMenuControllerDelegate>)delegate{
    _delegate = delegate;
    _menuFlags.respondsToWillShowViewController = [_delegate respondsToSelector:@selector(menuController:willShowViewController:)];
}

- (void)setRightController:(UIViewController *)rightController{
    if (rightController != _rightController) {
        [_rightController release];
        _rightController = [rightController retain];
        _menuFlags.canShowRight = (_rightController != nil);
    }
}

- (void)setLeftController:(UIViewController *)leftController{
    if (leftController != _leftController) {
        [_leftController release];
        _leftController = [leftController retain];
        _menuFlags.canShowLeft = (_leftController != nil);
    }    
}

- (void)setRootController:(UIViewController *)rootController{
    if (_rootController != rootController) {
        UIViewController *tempRoot = _rootController;
        _rootController = [rootController retain];
        if (tempRoot) {
            [tempRoot.view removeFromSuperview];
            [tempRoot release];
            tempRoot = nil;
        }
        
        if (_rootController) {
            UIView *rootView = _rootController.view;
            rootView.frame = self.view.bounds;
            [self.view addSubview:rootView];
            [rootView addGestureRecognizer:self.tap];
            [rootView addGestureRecognizer:self.pan];
        }
        
    }
    
}

#pragma mark - -- pan 

- (void)pan:(UIPanGestureRecognizer *)gesture{
    UIGestureRecognizerState gestureState = gesture.state;
    if (gestureState == UIGestureRecognizerStateBegan) {
        _panOriginX = self.rootController.view.frame.origin.x;
        _panVelocity = CGPointZero;
        
        if ([gesture velocityInView:self.view].x > 0) {
            _panDirection = DDMenuPanDirectionRight;
        } else {
            _panDirection = DDMenuPanDirectionLeft;
        }
        
    } else if (gestureState == UIGestureRecognizerStateChanged){
        CGPoint velocity = [gesture velocityInView:self.view];
        if (velocity.x * _panVelocity.x + velocity.y * _panVelocity.y < 0) { //向量乘积 小于零反向，
            _panDirection = (_panDirection == DDMenuPanDirectionLeft) ? DDMenuPanDirectionRight : DDMenuPanDirectionLeft;
        }
        
        _panVelocity = velocity;
        CGPoint translation = [gesture translationInView:self.view];
        CGRect frame = _rootController.view.frame;
        frame.origin.x = _panOriginX + translation.x;
        
        // show left or right before set _root frame
        if (frame.origin.x > 0.0f && !_menuFlags.showingLeftView) {
            if (_menuFlags.showingRightView) {
                _menuFlags.showingRightView = NO;
                [self.rightController.view removeFromSuperview];
            }
            
            if (_menuFlags.canShowLeft) {
                _menuFlags.showingLeftView = YES;
                
                CGRect frameLeft = self.view.bounds;
                frameLeft.size.width = kMenuFullWidthLeft;
                self.leftController.view.frame = frameLeft;
                [self.view insertSubview:self.leftController.view atIndex:0];
            } else {
                frame.origin.x = 0.0f;
            }
        } else if (frame.origin.x < 0.0f && !_menuFlags.showingRightView){
            if (_menuFlags.showingLeftView) {
                _menuFlags.showingLeftView = NO;
                [self.leftController.view removeFromSuperview];
            }
            
            if (_menuFlags.canShowRight) {
                _menuFlags.showingRightView = YES;
                
                CGRect frameRight = self.view.bounds;
                frameRight.origin.x += frameRight.size.width - kMenuFullWidthRight;
                frameRight.size.width = kMenuFullWidthRight;
                self.rightController.view.frame = frameRight;
                [self.view insertSubview:self.rightController.view atIndex:0];
                
            } else {
                frame.origin.x = 0.0f;
            }
        }
        
        _rootController.view.frame = frame;
        
    } else if (gestureState == UIGestureRecognizerStateEnded || gestureState == UIGestureRecognizerStateCancelled){
        _rootController.view.superview.userInteractionEnabled = NO;
        
        DDMenuPanCompletion completion = DDMenuPanCompletionRoot;
        if (_panDirection == DDMenuPanDirectionRight && _menuFlags.showingLeftView) {
            completion = DDMenuPanCompletionLeft;
        } else if (_panDirection == DDMenuPanDirectionLeft && _menuFlags.showingRightView){
            completion = DDMenuPanCompletionRight;
        }
        
//        CGPoint locationPoint = [gesture locationInView:self.view];
        CGPoint velocity = [gesture velocityInView:self.view];
        CGFloat currentRootX = _rootController.view.frame.origin.x;
        velocity.x = ABS(velocity.x);
        
        BOOL bounce = velocity.x > 800;
        
        if (!bounce) {
            
            if (_prePosition == PreDDMenuPanStatePositionRoot) {
                if (currentRootX > 0) {
                    if (currentRootX > kMenuSliderTiggerWidth) {
                        completion = DDMenuPanCompletionLeft;
                    } else {
                        completion = DDMenuPanCompletionRoot;
                        NSLog(@"to");
                    }
                } else {
                    if (currentRootX < -kMenuSliderTiggerWidth) {
                        completion = DDMenuPanCompletionRight;
                    } else {
                        completion = DDMenuPanCompletionRoot;
                    }
                }
                
                
            } else if (_prePosition == PreDDMenuPanStatePositionLeft) {
                if (currentRootX < self.view.bounds.size.width - kMenuSliderTiggerWidth) {
                    completion = DDMenuPanCompletionRoot;
                } else {
                    completion = DDMenuPanCompletionLeft;
                }
            } else {
                if (currentRootX < kMenuSliderTiggerWidth - self.view.bounds.size.width) {
                    completion = DDMenuPanCompletionRight;
                } else {
                    completion = DDMenuPanCompletionRoot;
                }
            }
        }
        
        NSLog(@"complete1  = %d", completion);
        
        CGFloat originX = _rootController.view.frame.origin.x;
        CGFloat width = _rootController.view.frame.size.width;
        CGFloat span = (width - kMenuOverlayWidth);// 320-40
        CGFloat duration = kMenuSlideDuration;
        
        if (bounce) {
            duration = (span / velocity.x);
        } else {
            duration = ((span - originX) / span) * duration;
        }
        
        __block typeof(self) blockSelf = self;
        [CATransaction begin];
        [CATransaction setCompletionBlock:^{
            [blockSelf panAnimationDidFinishedCallback:completion];
        }];
        
        CGPoint pos = _rootController.view.layer.position;
        CAKeyframeAnimation *animation = [CAKeyframeAnimation animationWithKeyPath:@"position"];
        
        NSMutableArray *keyTimes = [[NSMutableArray alloc] initWithCapacity:bounce ? 3 : 2];
        NSMutableArray *values = [[NSMutableArray alloc] initWithCapacity:bounce ? 3 : 2];
        NSMutableArray *timingFunctions = [[NSMutableArray alloc] initWithCapacity:bounce ? 3 : 2];
        
        [values addObject:[NSValue valueWithCGPoint:pos]];
        [timingFunctions addObject:[CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseIn]];
        [keyTimes addObject:[NSNumber numberWithFloat:0.0f]];
        if (bounce) {
            
            duration += kMenuBounceDuration;
            [keyTimes addObject:[NSNumber numberWithFloat:1.0f - ( kMenuBounceDuration / duration)]];
            if (completion == DDMenuPanCompletionLeft) {
                
                [values addObject:[NSValue valueWithCGPoint:CGPointMake(((width/2) + span) + kMenuBounceOffset, pos.y)]];
                
            } else if (completion == DDMenuPanCompletionRight) {
                
                [values addObject:[NSValue valueWithCGPoint:CGPointMake(-((width/2) - (kMenuOverlayWidth-kMenuBounceOffset)), pos.y)]];
                
            } else {
                
                // depending on which way we're panning add a bounce offset
                if (_panDirection == DDMenuPanDirectionLeft) {
                    [values addObject:[NSValue valueWithCGPoint:CGPointMake((width/2) - kMenuBounceOffset, pos.y)]];
                } else {
                    [values addObject:[NSValue valueWithCGPoint:CGPointMake((width/2) + kMenuBounceOffset, pos.y)]];
                }
                
            }
            
            [timingFunctions addObject:[CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut]];
            
        }
        
        if (completion == DDMenuPanCompletionLeft) {
            [values addObject:[NSValue valueWithCGPoint:CGPointMake((width/2) + span, pos.y)]];
        } else if (completion == DDMenuPanCompletionRight) {
            [values addObject:[NSValue valueWithCGPoint:CGPointMake(-((width/2) - kMenuOverlayWidth), pos.y)]];
        } else {
            [values addObject:[NSValue valueWithCGPoint:CGPointMake(width/2, pos.y)]];
        }
        
        [timingFunctions addObject:[CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut]];
        [keyTimes addObject:[NSNumber numberWithFloat:1.0f]];
        
        animation.timingFunctions = timingFunctions;
        animation.keyTimes = keyTimes;
        //animation.calculationMode = @"cubic";
        animation.values = values;
        animation.removedOnCompletion = NO;
        animation.fillMode = kCAFillModeForwards;
        [_rootController.view.layer addAnimation:animation forKey:nil];
        [CATransaction commit];
    }
}

- (void)tap:(UIPanGestureRecognizer *)gesture{

    gesture.enabled = NO;
    [self showRootController:YES];
}

#pragma mark - show

- (void)showLeftController:(BOOL)animated duration:(NSTimeInterval)interval{
    if (!_menuFlags.canShowLeft) return;
    
    if (_rightController && _rightController.view.superview) {
        [_rightController.view removeFromSuperview];
        _menuFlags.showingRightView = NO;
    }
    
    if (_menuFlags.respondsToWillShowViewController) {
        [self.delegate menuController:self willShowViewController:self.leftController];
    }
    
    _menuFlags.showingLeftView = YES;
    _rootController.view.superview.userInteractionEnabled = NO;
    
    UIView *view = self.leftController.view;
	CGRect frame = self.view.bounds;
	frame.size.width = kMenuFullWidthLeft;
    view.frame = frame;
    [self.view insertSubview:view atIndex:0];
    [self.leftController viewWillAppear:animated];
    
    frame = _rootController.view.frame;
    frame.origin.x = CGRectGetMaxX(view.frame) - (kMenuFullWidthLeft - kMenuDisplayedWidth);
    
    if (animated) {
        [UIView beginAnimations:@"showLeft" context:nil];
        [UIView setAnimationDuration:interval];
        [UIView setAnimationDelegate:self];
        [UIView setAnimationDidStopSelector:@selector(animationDidStop:finished:context:)];
    }
    
    _rootController.view.frame = frame;
    [self showShadow:YES];
    if (!animated) {
        [self animationDidStop:@"showLeft" finished:nil context:nil];
    }
    
    if (animated) {
        [UIView commitAnimations];
    }
    
}

- (void)showLeftController:(BOOL)animated{
    [self showLeftController:animated duration:self.duration];
}

- (void)showRightController:(BOOL)animated duration:(NSTimeInterval)interval{
    if (!_menuFlags.canShowRight) return;
    
    if (_leftController && _leftController.view.superview) {
        [_leftController.view removeFromSuperview];
        _menuFlags.showingLeftView = NO;
    }
    
    if (_menuFlags.respondsToWillShowViewController) {
        [self.delegate menuController:self willShowViewController:self.rightController];
    }
    
    _menuFlags.showingRightView = YES;
    _rootController.view.superview.userInteractionEnabled = NO;
    
    UIView *view = self.rightController.view;
    CGRect frame = self.view.bounds;
	frame.origin.x += frame.size.width - kMenuFullWidthRight;
	frame.size.width = kMenuFullWidthRight;
    view.frame = frame;
    [self.view insertSubview:view atIndex:0];
    
    frame = _rootController.view.frame;
    frame.origin.x = -(frame.size.width - kMenuOverlayWidth);
    
    if (animated) {
        [UIView beginAnimations:@"showRight" context:nil];
        [UIView setAnimationDuration:interval];
        [UIView setAnimationDelegate:self];
        [UIView setAnimationDidStopSelector:@selector(animationDidStop:finished:context:)];
    }
    
    _rootController.view.frame = frame;
    [self showShadow:YES];
    if (!animated) {
        [self animationDidStop:@"showRight" finished:nil context:nil];
    }
    
    if (animated) {
        [UIView commitAnimations];
    }
}

- (void)showRightController:(BOOL)animated{
    [self showRightController:animated duration:self.duration];
}

- (void)showRootController:(BOOL)animated duration:(NSTimeInterval)interval{
    [_tap setEnabled:NO];
    _rootController.view.userInteractionEnabled = YES;
    
    _rootController.view.superview.userInteractionEnabled = NO;
    CGRect frame = _rootController.view.frame;
    frame.origin.x = 0.0f;
    
    if (animated) {
        [UIView beginAnimations:@"showRoot" context:nil];
        [UIView setAnimationDuration:interval];
        [UIView setAnimationDelegate:self];
        [UIView setAnimationDidStopSelector:@selector(animationDidStop:finished:context:)];
    }
    
    _rootController.view.frame = frame;
    
    if (animated) {
        [UIView commitAnimations];
    }
    
}

- (void)showRootController:(BOOL)animated{
    [self showRootController:animated duration:self.duration];
}

#pragma mark - UIGesture Delegate

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch{
    
    return YES;
}

#pragma mark - 
- (void)panAnimationDidFinishedCallback:(DDMenuPanCompletion)completion{
    if (completion == DDMenuPanCompletionLeft) {
        [self showLeftController:NO];
        
    } else if (completion == DDMenuPanCompletionRight){
        [self showRightController:NO];
    } else{
        [self showRootController:NO];
    }
    _prePosition = (PreDDMenuPanStatePosition)completion;
    [_rootController.view.layer removeAllAnimations];
    _rootController.view.superview.userInteractionEnabled = YES;
}

- (void)animationDidStop:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context{
    _rootController.view.superview.userInteractionEnabled = YES;
    if ([animationID isEqualToString:@"showLeft"]) {
        _tap.enabled = YES;
        _maskPan.enabled = YES;
        panMaskView.hidden = NO;
    } else if ([animationID isEqualToString:@"showRight"]){
        _tap.enabled = YES;
        _maskPan.enabled = YES;
        panMaskView.hidden = NO;
    } else if ([animationID isEqualToString:@"showRoot"]){
        if (_leftController && _leftController.view.superview) {
            [_leftController.view removeFromSuperview];
        }
        
        if (_rightController && _rightController.view.superview) {
            [_rightController.view removeFromSuperview];
        }
        
        [self showShadow:NO];
        _menuFlags.showingLeftView = NO;
        _menuFlags.showingRightView = NO;
        _maskPan.enabled = NO;
        _pan.enabled = YES;
        panMaskView.hidden = YES;
    }
}

@end
